@@ -170,7 +170,7 @@ static int zx_scan_elem_end(struct zx_ctx* c, const char* start, const char* fun
   ++c->p;
   name = c->p;
   ZX_LOOK_FOR(c,'>');
-  if (memcmp(start, name, c->p-name))	{
+  if (memcmp(start?start:"", name, c->p-name))	{
     errloc = MAX(c->p - 20, c->bas);
     ERR("%s: Mismatching close tag(%.*s) pos=%d (%.*s)", func, c->p-name, name, c->p - c->bas, MIN(c->lim - errloc, 40), errloc);
     ++c->p;
@@ -221,7 +221,7 @@ static int zx_chk_el_ord(struct zx_elem_s* x)
   return 0;
 }
 
-/*() Insert an attribute to element's attribtue list in canoncically sorted
+/*() Insert an attribute to element's attribute list in canoncically sorted
  * place, i.e. no namespace sorts first, namespaced attribute sort by
  * namespace URI (not namespace prefix). Assumes the attribute
  * list has so far been sorted. Used as part of insertion sort. */
