@@ -245,8 +245,10 @@ int zxsig_validate(struct zx_ctx* c, X509* cert, struct zx_ds_Signature_s* sig,
       if (!ssref->sref->Transforms)
 	continue;
       for (xform = ssref->sref->Transforms->Transform;
-	   xform && xform->gg.g.tok == zx_ds_Transform_ELEM;
+	   xform;
 	   xform = (void*)xform->gg.g.n) {
+	if (xform->gg.g.tok != zx_ds_Transform_ELEM)
+	  continue;
 	ss = xform->InclusiveNamespaces ? &xform->InclusiveNamespaces->PrefixList->g : 0;
 	if (!ss || !ss->len)
 	  continue;
